"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.evalExpr = exports.autorunForDebugExpr = exports.debugExprMap = void 0;
var tslib_1 = require("tslib");
var cals_1 = require("@cloudbase/cals");
var mobx_1 = require("mobx");
var eventProxy_1 = require("./eventProxy");
var generate_1 = require("./adapter/generate");
var lodash_es_1 = require("lodash-es");
/**
 * 以 widget id 为 key，只记录同个 widget 的最后一次表达式的结果
 * 这里因为同时只会执行（调试）一个表达式，所以只需保存最后一次执行的结果即可
 */
var eventProxy = new eventProxy_1.EventProxy();
function debounceFnFactory(fn, wait) {
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    var id = "".concat(fn.name, ":").concat(JSON.stringify(args));
    debounceFnFactory[id] = debounceFnFactory[id] || (0, lodash_es_1.debounce)(fn.bind.apply(fn, tslib_1.__spreadArray([this], tslib_1.__read(args), false)), wait);
    return debounceFnFactory[id];
}
exports.debugExprMap = (0, mobx_1.observable)({});
function autorunForDebugExpr(_a) {
    var widgetId = _a.widgetId, subForItems = _a.subForItems, globalCtx = _a.globalCtx, getDataContextFn = _a.getDataContextFn, get$wFn = _a.get$wFn, get$scopeFn = _a.get$scopeFn;
    var forDebugExprDispose = (0, mobx_1.autorun)(function () {
        // FIXME: 暂时没考虑低码组件开发模式
        var dataContext = (0, mobx_1.untracked)(getDataContextFn);
        var $w = (0, mobx_1.untracked)(get$wFn);
        var $scope = (0, mobx_1.untracked)(get$scopeFn);
        (0, mobx_1.untracked)(function () {
            if (!exports.debugExprMap[widgetId]) {
                exports.debugExprMap[widgetId] = {
                    expr: '',
                    result: ''
                };
            }
        });
        var expr = exports.debugExprMap[widgetId].expr;
        (0, mobx_1.untracked)(function () {
            var _a;
            exports.debugExprMap[widgetId].result = ''; // 先清空，这样如果下边失败了也不会有错误的值
            var fn = (_a = (0, generate_1.generateDataBinds)(globalCtx, [
                {
                    type: cals_1.PropBindType.expression,
                    bindDataPath: expr,
                    propertyPath: 'expr',
                },
            ])) === null || _a === void 0 ? void 0 : _a['expr'];
            if (fn) {
                var result = fn(subForItems, undefined, dataContext, $w, $scope);
                exports.debugExprMap[widgetId].result = result;
            }
            // 这样对于数组只会拿最后一项
            debounceFnFactory(function (widgetId) { return eventProxy.emit('done_' + widgetId); }, 300, widgetId)();
        });
    });
    return forDebugExprDispose;
}
exports.autorunForDebugExpr = autorunForDebugExpr;
function evalExpr(widgetId, expr) {
    return new Promise(function (resolve) {
        if (!exports.debugExprMap[widgetId])
            resolve(undefined);
        if (exports.debugExprMap[widgetId].expr === expr) {
            // 表达式没变，直接返回结果
            resolve(exports.debugExprMap[widgetId].result);
        }
        else {
            eventProxy.once('done_' + widgetId, function () {
                resolve(exports.debugExprMap[widgetId].result);
            });
            // 触发 autorun
            exports.debugExprMap[widgetId].expr = expr;
        }
    });
}
exports.evalExpr = evalExpr;
window['__testExpr'] = evalExpr;
window['__debugExprMap'] = exports.debugExprMap;
//# sourceMappingURL=debugExpr.js.map